@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
+ * Copyright (C) 2007 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  * 
  * Authors: Martyn Russell <martyn@imendio.com>
+ *          Xavier Claessens <xclaesse@gmail.com>
  */
 
 #include "config.h"
@@ -40,18 +42,18 @@
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_ACCOUNT_CHOOSER, EmpathyAccountChooserPriv))
 
 typedef struct {
-	MissionControl   *mc;
-	McAccountMonitor *monitor;
-
-	gboolean          set_active_item;
-	gboolean          can_select_all;
-	gboolean          has_all_option;
+	MissionControl                 *mc;
+	McAccountMonitor               *monitor;
+	gboolean                        set_active_item;
+	gboolean                        has_all_option;
+	EmpathyAccountChooserFilterFunc filter;
+	gpointer                        filter_data;
 } EmpathyAccountChooserPriv;
 
 typedef struct {
 	EmpathyAccountChooser *chooser;
-	McAccount            *account;
-	gboolean              set;
+	McAccount             *account;
+	gboolean               set;
 } SetAccountData;
 
 enum {
@@ -83,14 +85,13 @@ static void     account_chooser_account_deleted_cb     (McAccountMonitor
 static void     account_chooser_account_remove_foreach (McAccount                       *account,
 							EmpathyAccountChooser            *chooser);
 static void     account_chooser_update_iter            (EmpathyAccountChooser            *chooser,
-							GtkTreeIter                     *iter,
-							McAccount                       *account);
+							GtkTreeIter                     *iter);
 static void     account_chooser_status_changed_cb      (MissionControl                  *mc,
 							TelepathyConnectionStatus        status,
 							McPresence                       presence,
 							TelepathyConnectionStatusReason  reason,
 							const gchar                     *unique_name,
-							EmpathyAccountChooser            *chooser);
+							EmpathyAccountChooser           *chooser);
 static gboolean account_chooser_separator_func         (GtkTreeModel                    *model,
 							GtkTreeIter                     *iter,
 							EmpathyAccountChooser            *chooser);
@@ -98,14 +99,9 @@ static gboolean account_chooser_set_account_foreach    (GtkTreeModel
 							GtkTreePath                     *path,
 							GtkTreeIter                     *iter,
 							SetAccountData                  *data);
-static gboolean account_chooser_set_enabled_foreach    (GtkTreeModel                    *model,
-							GtkTreePath                     *path,
-							GtkTreeIter                     *iter,
-							EmpathyAccountChooser            *chooser);
 
 enum {
 	PROP_0,
-	PROP_CAN_SELECT_ALL,
 	PROP_HAS_ALL_OPTION,
 };
 
@@ -120,14 +116,6 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 	object_class->get_property = account_chooser_get_property;
 	object_class->set_property = account_chooser_set_property;
 
-	g_object_class_install_property (object_class,
-					 PROP_CAN_SELECT_ALL,
-					 g_param_spec_boolean ("can-select-all",
-							       "Can Select All",
-							       "Should the user be able to select offline accounts",
-							       FALSE,
-							       G_PARAM_READWRITE));
-
 	g_object_class_install_property (object_class,
 					 PROP_HAS_ALL_OPTION,
 					 g_param_spec_boolean ("has-all-option",
@@ -142,6 +130,11 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 static void
 empathy_account_chooser_init (EmpathyAccountChooser *chooser)
 {
+	EmpathyAccountChooserPriv *priv = GET_PRIV (chooser);
+
+	priv->set_active_item = FALSE;
+	priv->filter = NULL;
+	priv->filter_data = NULL;
 }
 
 static void
@@ -180,9 +173,6 @@ account_chooser_get_property (GObject    *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_CAN_SELECT_ALL:
-		g_value_set_boolean (value, priv->can_select_all);
-		break;
 	case PROP_HAS_ALL_OPTION:
 		g_value_set_boolean (value, priv->has_all_option);
 		break;
@@ -203,10 +193,6 @@ account_chooser_set_property (GObject      *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_CAN_SELECT_ALL:
-		empathy_account_chooser_set_can_select_all (EMPATHY_ACCOUNT_CHOOSER (object),
-							   g_value_get_boolean (value));
-		break;
 	case PROP_HAS_ALL_OPTION:
 		empathy_account_chooser_set_has_all_option (EMPATHY_ACCOUNT_CHOOSER (object),
 							   g_value_get_boolean (value));
@@ -269,7 +255,7 @@ empathy_account_chooser_get_account (EmpathyAccountChooser *chooser)
 
 gboolean
 empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
-				    McAccount            *account)
+				     McAccount             *account)
 {
 	GtkComboBox    *combobox;
 	GtkTreeModel   *model;
@@ -292,46 +278,6 @@ empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
 	return data.set;
 }
 
-gboolean
-empathy_account_chooser_get_can_select_all (EmpathyAccountChooser *chooser)
-{
-	EmpathyAccountChooserPriv *priv;
-
-	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), FALSE);
-
-	priv = GET_PRIV (chooser);
-	
-	return priv->can_select_all;
-}
-
-void
-empathy_account_chooser_set_can_select_all (EmpathyAccountChooser *chooser,
-					   gboolean              can_select_all)
-{
-	EmpathyAccountChooserPriv *priv;
-	GtkComboBox              *combobox;
-	GtkTreeModel             *model;
-
-	g_return_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser));
-
-	priv = GET_PRIV (chooser);
-
-	if (priv->can_select_all == can_select_all) {
-		return;
-	}
-
-	combobox = GTK_COMBO_BOX (chooser);
-	model = gtk_combo_box_get_model (combobox);
-
-	priv->can_select_all = can_select_all;
-
-	gtk_tree_model_foreach (model,
-				(GtkTreeModelForeachFunc) account_chooser_set_enabled_foreach,
-				chooser);
-
-	g_object_notify (G_OBJECT (chooser), "can-select-all");
-}
-
 gboolean
 empathy_account_chooser_get_has_all_option (EmpathyAccountChooser *chooser)
 {
@@ -426,7 +372,7 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 	gtk_cell_layout_clear (GTK_CELL_LAYOUT (combobox));
 
 	store = gtk_list_store_new (COL_ACCOUNT_COUNT,
-				    G_TYPE_STRING,
+				    G_TYPE_STRING,    /* Image */
 				    G_TYPE_STRING,    /* Name */
 				    G_TYPE_BOOLEAN,   /* Enabled */
 				    MC_TYPE_ACCOUNT);
@@ -471,21 +417,22 @@ account_chooser_account_created_cb (McAccountMonitor     *monitor,
 }
 
 static void
-account_chooser_account_add_foreach (McAccount            *account,
+account_chooser_account_add_foreach (McAccount             *account,
 				     EmpathyAccountChooser *chooser)
 {
-	EmpathyAccountChooserPriv *priv;
-	GtkListStore             *store;
-	GtkComboBox              *combobox;
-	GtkTreeIter               iter;
-
-	priv = GET_PRIV (chooser);
+	GtkListStore *store;
+	GtkComboBox  *combobox;
+	GtkTreeIter   iter;
+	gint          position;
 
 	combobox = GTK_COMBO_BOX (chooser);
 	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
 
-	gtk_list_store_append (store, &iter);
-	account_chooser_update_iter (chooser, &iter, account);
+	position = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);
+	gtk_list_store_insert_with_values (store, &iter, position,
+					   COL_ACCOUNT_POINTER, account,
+					   -1);
+	account_chooser_update_iter (chooser, &iter);
 }
 
 static void
@@ -500,43 +447,102 @@ account_chooser_account_deleted_cb (McAccountMonitor     *monitor,
 	g_object_unref (account);
 }
 
+typedef struct {
+	McAccount   *account;
+	GtkTreeIter *iter;
+	gboolean     found;
+} FindAccountData;
+
+static gboolean
+account_chooser_find_account_foreach (GtkTreeModel *model,
+				      GtkTreePath  *path,
+				      GtkTreeIter  *iter,
+				      gpointer      user_data)
+{
+	FindAccountData *data = user_data;
+	McAccount       *account;
+
+	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
+
+	if (empathy_account_equal (account, data->account)) {
+		data->found = TRUE;
+		*(data->iter) = *iter;
+		g_object_unref (account);
+
+		return TRUE;
+	}
+
+	g_object_unref (account);
+
+	return FALSE;
+}
+
+static gboolean
+account_chooser_find_account (EmpathyAccountChooser *chooser,
+			      McAccount             *account,
+			      GtkTreeIter           *iter)
+{
+	GtkListStore    *store;
+	GtkComboBox     *combobox;
+	FindAccountData  data;
+
+	combobox = GTK_COMBO_BOX (chooser);
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
+
+	data.account = account;
+	data.iter = iter;
+	gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+				account_chooser_find_account_foreach,
+				&data);
+
+	return data.found;
+}
+
 static void
 account_chooser_account_remove_foreach (McAccount            *account,
 					EmpathyAccountChooser *chooser)
 {
-	/* Fixme: TODO */
+	GtkListStore *store;
+	GtkComboBox  *combobox;
+	GtkTreeIter   iter;
+
+	combobox = GTK_COMBO_BOX (chooser);
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
+
+	if (account_chooser_find_account (chooser, account, &iter)) {
+		gtk_list_store_remove (store, &iter);
+	}
 }
 
 static void
 account_chooser_update_iter (EmpathyAccountChooser *chooser,
-			     GtkTreeIter          *iter,
-			     McAccount            *account)
+			     GtkTreeIter           *iter)
 {
 	EmpathyAccountChooserPriv *priv;
-	GtkListStore             *store;
-	GtkComboBox              *combobox;
-	TpConn                   *tp_conn;
-	const gchar              *icon_name;
-	gboolean                  is_enabled;
+	GtkListStore              *store;
+	GtkComboBox               *combobox;
+	McAccount                 *account;
+	const gchar               *icon_name;
+	gboolean                   is_enabled = TRUE;
 
 	priv = GET_PRIV (chooser);
 
 	combobox = GTK_COMBO_BOX (chooser);
 	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
 
-	icon_name = empathy_icon_name_from_account (account);
-	tp_conn = mission_control_get_connection (priv->mc, account, NULL);
-	is_enabled = (tp_conn != NULL || priv->can_select_all);
+	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+			    COL_ACCOUNT_POINTER, &account,
+			    -1);
 
-	if (tp_conn) {
-		g_object_unref (tp_conn);
+	icon_name = empathy_icon_name_from_account (account);
+	if (priv->filter) {
+		is_enabled = priv->filter (account, priv->filter_data);
 	}
 
 	gtk_list_store_set (store, iter,
 			    COL_ACCOUNT_IMAGE, icon_name,
 			    COL_ACCOUNT_TEXT, mc_account_get_display_name (account),
 			    COL_ACCOUNT_ENABLED, is_enabled,
-			    COL_ACCOUNT_POINTER, account,
 			    -1);
 
 	/* set first connected account as active account */
@@ -544,6 +550,8 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 		priv->set_active_item = TRUE;
 		gtk_combo_box_set_active_iter (combobox, iter);
 	}
+
+	g_object_unref (account);
 }
 
 static void
@@ -552,9 +560,16 @@ account_chooser_status_changed_cb (MissionControl                  *mc,
 				   McPresence                       presence,
 				   TelepathyConnectionStatusReason  reason,
 				   const gchar                     *unique_name,
-				   EmpathyAccountChooser            *chooser)
+				   EmpathyAccountChooser           *chooser)
 {
-	/* FIXME: implement */
+	McAccount   *account;
+	GtkTreeIter  iter;
+
+	account = mc_account_lookup (unique_name);
+	if (account_chooser_find_account (chooser, account, &iter)) {
+		account_chooser_update_iter (chooser, &iter);
+	}
+	g_object_unref (account);
 }
 
 static gboolean
@@ -614,24 +629,49 @@ account_chooser_set_account_foreach (GtkTreeModel   *model,
 }
 
 static gboolean
-account_chooser_set_enabled_foreach (GtkTreeModel         *model,
-				     GtkTreePath          *path,
-				     GtkTreeIter          *iter,
-				     EmpathyAccountChooser *chooser)
+account_chooser_filter_foreach (GtkTreeModel *model,
+				GtkTreePath  *path,
+				GtkTreeIter  *iter,
+				gpointer      chooser)
 {
-	EmpathyAccountChooserPriv *priv;
-	McAccount                *account;
-
-	priv = GET_PRIV (chooser);
-
-	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
-	if (!account) {
-		return FALSE;
-	}
-
-	account_chooser_update_iter (chooser, iter, account);
-	g_object_unref (account);
-
+	account_chooser_update_iter (chooser, iter);
 	return FALSE;
 }
 
+void
+empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
+                                    EmpathyAccountChooserFilterFunc  filter,
+                                    gpointer                         user_data)
+{
+	EmpathyAccountChooserPriv *priv;
+	GtkTreeModel *model;
+
+	g_return_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser));
+
+	priv = GET_PRIV (chooser);
+
+	priv->filter = filter;
+	priv->filter_data = user_data;
+
+	/* Refilter existing data */
+	priv->set_active_item = FALSE;
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (chooser));
+	gtk_tree_model_foreach (model, account_chooser_filter_foreach, chooser);
+}
+
+gboolean
+empathy_account_chooser_filter_is_connected (McAccount *account,
+					     gpointer   user_data)
+{
+	MissionControl            *mc;
+	TelepathyConnectionStatus  status;
+
+	g_return_val_if_fail (MC_IS_ACCOUNT (account), FALSE);
+
+	mc = empathy_mission_control_new ();
+	status = mission_control_get_connection_status (mc, account, NULL);
+	g_object_unref (mc);
+
+	return status == TP_CONN_STATUS_CONNECTED;
+}
+
