@@ -3,20 +3,19 @@
  * Copyright (C) 2007 Xavier Claessens <xclaesse@gmail.com>
  * Copyright (C) 2007 Collabora Ltd.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
@@ -348,7 +347,7 @@ tp_contact_list_newchannel_cb (DBusGProxy           *proxy,
 			       const gchar          *channel_type,
 			       TelepathyHandleType   handle_type,
 			       guint                 channel_handle,
-			       gboolean              suppress_handle,
+			       gboolean              suppress_handler,
 			       EmpathyTpContactList *list)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
@@ -357,7 +356,7 @@ tp_contact_list_newchannel_cb (DBusGProxy           *proxy,
 	const gchar              *bus_name;
 
 	if (strcmp (channel_type, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST) != 0 ||
-	    suppress_handle) {
+	    suppress_handler) {
 		return;
 	}
 
@@ -818,6 +817,10 @@ tp_contact_list_get_all_groups (EmpathyContactList *list)
 
 	g_return_val_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list), NULL);
 
+	if (priv->protocol_group) {
+		groups = g_list_prepend (groups, g_strdup (priv->protocol_group));
+	}
+
 	for (l = priv->groups; l; l = l->next) {
 		const gchar *name;
 
@@ -838,15 +841,20 @@ tp_contact_list_get_groups (EmpathyContactList *list,
 
 	g_return_val_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list), NULL);
 
+	if (priv->protocol_group) {
+		ret = g_list_prepend (ret, g_strdup (priv->protocol_group));
+	}
+
 	groups = g_hash_table_lookup (priv->contacts_groups, contact);
 	if (!groups) {
-		return NULL;
+		return ret;
 	}
 
 	for (l = *groups; l; l = l->next) {
 		ret = g_list_prepend (ret, g_strdup (l->data));
 	}
 
+
 	return ret;
 }
 
@@ -887,7 +895,7 @@ tp_contact_list_get_group (EmpathyTpContactList *list,
 				      TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
 				      TP_HANDLE_TYPE_GROUP,
 				      handle,
-				      FALSE,
+				      TRUE,
 				      &object_path,
 				      &error)) {
 		empathy_debug (DEBUG_DOMAIN,
